from __future__ import print_function

import sympy as sp
sp.init_printing()
dphi, dpsi, dtheta = sp.var("phi_d psi_d theta_d")
phi, psi, theta = sp.var("phi psi theta")
x, y, z = sp.var("x y z")
qx, qy, qz = sp.var("qx qy qz")
qa, qb, qc = sp.var("qa qb qc")

def Rx(a):
    return sp.Matrix([[1, 0, 0], [0, sp.cos(a), sp.sin(a)], [0, -sp.sin(a), sp.cos(a)]])
def Ry(a):
    return sp.Matrix([[sp.cos(a), 0, sp.sin(a)], [0, 1, 0], [-sp.sin(a), 0, sp.cos(a)]])
def Rz(a):
    return sp.Matrix([[sp.cos(a), sp.sin(a), 0], [-sp.sin(a), sp.cos(a), 0], [0, 0, 1]])

print("==== asymmetric ====")
q_xy = sp.Matrix([[qx], [qy], [0]])
q_abc = sp.Matrix([[qa], [qb], [qc]])
p_xyz = sp.Matrix([[x], [y], [z]])
jitter = Rx(dphi)*Ry(dtheta)*Rz(dpsi)
view = Rz(phi)*Ry(theta)*Rz(psi)
view_inv = Rz(-psi)*Ry(-theta)*Rz(-phi)
print(">> jitter rotation")
for row in jitter*p_xyz: sp.pprint(row)
#print("\n>> jitter plus view")
#for row in view*jitter*p_xyz: sp.pprint(row)
print("\n>> view reverse")
for row in view_inv*q_xy: sp.pprint(row)
print("\n>> jitter reverse")
jitter_inv = Rz(-psi)*Ry(-theta)*Rx(-phi)
for row in jitter_inv*q_abc: sp.pprint(row)
print("\n>> jitter view reverse")
jitter_inv = Rz(-dpsi)*Ry(-dtheta)*Rx(-dphi)
for row in jitter_inv*view_inv*q_xy: sp.pprint(row)

print("\n\n==== symmetric ====")
q_x = sp.Matrix([[qx], [0], [0]])
q_ac = sp.Matrix([[qa], [0], [qc]])
p_xyz = sp.Matrix([[x], [y], [z]])
jitter = Rx(dphi)*Ry(dtheta)
view = Rz(phi)*Ry(theta)
jitter_inv = Ry(-dtheta)*Rx(-dphi)
view_inv = Ry(-theta)*Rz(-phi)
print(">> jitter rotation")
for row in jitter*p_xyz: sp.pprint(row)
#print("\n>> jitter plus view")
#for row in view*jitter*p_xyz: sp.pprint(row)
print("\n>> view reverse")
for row in view_inv*q_x: sp.pprint(row)
print("\n>> jitter reverse")
for row in jitter_inv*q_ac: sp.pprint(row)
